# -*- coding: utf-8 -*-
"""
@File    : crawler.py
@Time    : 2024/3/23 16:22:56
@Author  : lihao
@Contact : lihao2015@whu.edu.cn
"""

import os
import re
import logging
import argparse
import requests
from urllib.parse import urljoin, urlparse
from bs4 import BeautifulSoup
from typing import Dict, List, Optional
import aiohttp

try:
    from lh_tool.iterator import AsyncProcess, AsyncMultiProcess
except:
    from lh_tool.Iterator import AsyncProcess, AsyncMultiProcess


class Crawler:
    """
    Crawler

    Parameters:
        chunk_size(int): chunk size for download, default is 1024
        nprocs(int): nprocs for asynchronous multi-process download, default is 1, doesn't use multi-process
        concurrency(int): concurrency for asynchronous download, default is 0

    Example:
        .. code-block:: python

        url = "http://www.dili360.com/photo/column/6534.htm"
        name = "img"
        url_key = "src"
        pattern = "(.*.jpg)@!rw4"
        save_path = "images"
        crawler = Crawler()
        url_links = crawler.crawl(url, name, url_key, pattern)
        crawler.download(url_links, save_path)
    """

    def __init__(self, chunk_size=1024, nprocs=1, concurrency=0):
        self.chunk_size = chunk_size
        self.nprocs = nprocs
        self.concurrency = concurrency

    @staticmethod
    def check_url(url):
        """
        check url

        Args:
            url(str): url to check

        Returns:
            bool: True if url is valid, otherwise False
        """
        try:
            result = urlparse(url)
            return all([result.scheme, result.netloc])
        except Exception as e:
            logging.error(e)
            return False

    @staticmethod
    def parse_url(
        url: str,
        name: str,
        url_key: str,
        pattern: str = ".*",
        attrs: Optional[Dict[str, str]] = {},
    ) -> List[str]:
        """
        parse url

        Args:
            url(str): url to parse
            name(str): tag name to find
            url_key(str): key to find download url
            pattern(str): pattern to match, default is ".*"
            attrs(dict, optinal): attrs to find

        Returns:
            url_links(list): list of url link
        """
        url_links = set()

        # compile pattern
        new_attrs = {url_key: pattern}
        new_attrs.update(attrs)
        for key, value in new_attrs.items():
            new_attrs[key] = re.compile(value)
        pattern = new_attrs[url_key]

        try:
            html = requests.get(url).text
            soup = BeautifulSoup(html, "lxml")

            # find using attrs
            results = soup.find_all(name, new_attrs)
            for result in results:
                url_link = result.get(url_key)
                match = re.search(pattern, url_link)
                index = 1 if match.lastindex else 0
                url_link = match.group(index)
                if url_link:
                    url_links.add(url_link)

        except Exception as e:
            logging.error(e)

        return list(url_links)

    def crawl(
        self,
        url: str,
        name: str,
        url_key: str,
        pattern: str = ".*",
        attrs: Optional[Dict[str, str]] = {},
    ) -> List[str]:
        """
        crawl

        Args:
            url(str): url to crawl
            name(str): tag name to find
            url_key(str): key to find download url
            pattern(str): pattern to match, default is ".*"
            attrs(dict, optinal): attrs to find

        Returns:
            url_links(list): list of url link

        Exceprion:
            ValueError: when url is not valid
        """
        if not self.check_url(url):
            raise ValueError("url is not valid: {}".format(url))

        url_links = self.parse_url(url, name, url_key, pattern, attrs)
        url_links = [urljoin(url, url_link) for url_link in url_links]
        return url_links

    def download(self, url_links: List[str], save_path: str):
        """
        download

        Args:
            url_links(list[str]): list of url link to download
            save_path(str): path to save file

        Returns:
            save_file_list(list[str]): list of saved file
        """
        os.makedirs(save_path, exist_ok=True)
        save_file_list = [
            os.path.join(save_path, os.path.basename(url)) for url in url_links
        ]

        Process = AsyncMultiProcess if self.nprocs > 1 else AsyncProcess
        process = Process(
            self._download_single,
            nprocs=self.nprocs,
            concurrency=self.concurrency,
        )
        ret_list = process.run(url_links, save_file_list)
        logging.info(
            "total num: {}, finish num: {}".format(
                len(ret_list), sum(ret_list)
            )
        )
        save_file_list = [
            save_file
            for save_file, ret in zip(save_file_list, ret_list)
            if ret
        ]

        return save_file_list

    async def _download_single(self, url: str, save_file: str):
        """
        download single file asynchronously

        Args:
            url(str): url to download
            save_file(str): filename to save

        Returns:
            status(bool): download status
        """
        status = False
        try:
            async with aiohttp.ClientSession() as session:
                async with session.get(url) as res:
                    if res.status == 200:
                        with open(save_file, "wb") as f:
                            async for chunk in res.content.iter_chunked(
                                self.chunk_size
                            ):
                                f.write(chunk)
                            status = True
                    else:
                        logging.error(
                            "Failed to download file from {}. Status code: {}".format(
                                url, res.status
                            )
                        )
        except Exception as e:
            logging.error("Error occurred during download: {}".format(e))

        return status


def main():
    example = """
    example:
        crawler -u http://www.dili360.com/photo/column/6534.htm  -n img -k src -p "(.*.jpg)@\!rw4" -s images
    """
    parser = argparse.ArgumentParser(usage=example)
    parser.add_argument(
        "-u", "--url", type=str, help="url to crawl", required=True
    )
    parser.add_argument(
        "-n", "--name", type=str, help="tag name to find", required=True
    )
    parser.add_argument(
        "-k",
        "--url_key",
        type=str,
        help="key to find download url",
        required=True,
    )
    parser.add_argument(
        "-p",
        "--pattern",
        type=str,
        help="pattern to match, default is '.*'",
        default=".*",
    )
    parser.add_argument(
        "-s", "--save_path", type=str, help="save path", required=True
    )
    opts = parser.parse_args()
    print(opts)

    import time

    start_time = time.time()

    url = opts.url
    name = opts.name
    url_key = opts.url_key
    pattern = opts.pattern
    save_path = opts.save_path
    crawler = Crawler()
    url_links = crawler.crawl(url, name, url_key, pattern)
    crawler.download(url_links, save_path)

    end_time = time.time()
    logging.info("total time: {}".format(end_time - start_time))


if __name__ == "__main__":
    main()



# -*- encoding: utf-8 -*-
"""
@File    :   hf_download.py
@Time    :   2024/10/31 15:26:42
@Author  :   lihao57
@Version :   1.0
@Contact :   lihao57@baidu.com
@Reference:  https://github.com/LetheSec/HuggingFace-Download-Accelerator/blob/main/hf_download.py
"""


import os
import sys
import time
import logging
import argparse
import huggingface_hub


def download(
    args,
):
    """ """
    if args.use_hf_transfer:
        import hf_transfer

        # Enable hf-transfer if specified
        os.environ["HF_HUB_ENABLE_HF_TRANSFER"] = "1"
        logging.info(
            "export HF_HUB_ENABLE_HF_TRANSFER={}".format(
                os.getenv("HF_HUB_ENABLE_HF_TRANSFER")
            )
        )

    if args.model is None and args.dataset is None:
        logging.error("No model or dataset was specified.")
        sys.exit()
    elif args.model is not None and args.dataset is not None:
        logging.error("Only one model or dataset can be downloaded at a time.")
        sys.exit()

    if args.use_mirror:
        # Set default endpoint to mirror site if specified
        os.environ["HF_ENDPOINT"] = "https://hf-mirror.com"
        logging.info("export HF_ENDPOINT={}".format(os.getenv("HF_ENDPOINT")))

    if args.token is not None:
        token_option = f"--token {args.token}"
    else:
        token_option = ""

    if args.include is not None:
        include_option = f"--include {args.include}"
    else:
        include_option = ""

    if args.exclude is not None:
        exclude_option = f"--exclude {args.exclude}"
    else:
        exclude_option = ""

    options = args.options if args.options is not None else ""

    if args.model is not None:
        if args.save_dir is not None:
            save_path = os.path.join(args.save_dir, args.model)
            save_dir_option = f"--local-dir {save_path}"
        else:
            save_dir_option = ""

        download_shell = f"huggingface-cli download {token_option} {include_option} {exclude_option} {options} --repo-type model {args.model} {save_dir_option}"
        logging.info(download_shell)
        os.system(download_shell)

    elif args.dataset is not None:
        if args.save_dir is not None:
            save_path = os.path.join(args.save_dir, args.dataset)
            save_dir_option = f"--local-dir {save_path}"
        else:
            save_dir_option = ""

        download_shell = f"huggingface-cli download {token_option} {include_option} {exclude_option} {options} --repo-type dataset {args.dataset} {save_dir_option}"
        logging.info(download_shell)
        os.system(download_shell)


def main():
    # set base logging config
    fmt = "[%(asctime)s - %(levelname)s - %(filename)s:%(lineno)s] %(message)s"
    logging.basicConfig(format=fmt, level=logging.INFO)

    parser = argparse.ArgumentParser(
        description="HuggingFace Download Accelerator Script."
    )
    parser.add_argument(
        "-m",
        "--model",
        type=str,
        help="model name in huggingface, e.g., baichuan-inc/Baichuan2-7B-Chat",
    )
    parser.add_argument(
        "-d",
        "--dataset",
        type=str,
        help="dataset name in huggingface, e.g., zh-plus/tiny-imagenet",
    )
    parser.add_argument(
        "-t",
        "--token",
        type=str,
        help="hugging face access token for download meta-llama/Llama-2-7b-hf, e.g., hf_***** ",
    )
    parser.add_argument(
        "-i",
        "--include",
        type=str,
        help="Specify the file to be downloaded",
    )
    parser.add_argument(
        "-e",
        "--exclude",
        type=str,
        help="Files you don't want to download",
    )
    parser.add_argument(
        "-o",
        "--options",
        type=str,
        help="Other options",
    )
    parser.add_argument(
        "--save_dir",
        "-s",
        type=str,
        help="path to be saved after downloading.",
    )
    parser.add_argument(
        "--use_hf_transfer",
        default=True,
        type=eval,
        help="Use hf-transfer, default: True",
    )
    parser.add_argument(
        "--use_mirror",
        default=True,
        type=eval,
        help="Download from mirror, default: True",
    )
    args = parser.parse_args()
    logging.info(args)

    t1 = time.time()

    # run
    download(args)

    t2 = time.time()
    logging.info("time: {}".format(t2 - t1))


if __name__ == "__main__":
    main()



# -*- encoding: utf-8 -*-
"""
@File    :   play_image_web.py
@Time    :   2024/03/23 10:41:16
@Author  :   lihao57
@Version :   1.0
@Contact :   lihao57@baidu.com
"""


import os
import shutil
import json
import pywebio
import pywebio.output as output
import pywebio.pin as pin
import pywebio.session as session
from pywebio.platform.flask import webio_view
import logging
import time
import argparse
import socket
from flask import Flask, render_template
from lh_webtool.crawler import Crawler

app = Flask(__name__)
image_urls = []


def get_local_ip():
    """get local ip"""
    # 获取本机主机名
    hostname = socket.gethostname()
    # 获取本机 IP 地址
    local_ip = socket.gethostbyname(hostname)
    return local_ip


@app.route("/image")
def index():
    """index"""
    global image_urls
    return render_template("play_image.html", image_urls=image_urls)


class TaskManager:
    """
    TaskManager

    Parameterars:
        title (str): web title
        host (str): web host
        port (int): web port
    """

    def __init__(
        self,
        title,
        host,
        port,
    ):
        self.title = title
        self.host = host
        self.port = port

        # 获取当前执行脚本的绝对路径
        current_file = os.path.abspath(__file__)
        self.cache_path = os.path.join(os.path.dirname(current_file), "static")
        logging.info("cache path: {}".format(self.cache_path))
        self.crawler = Crawler()

        pywebio.config(title=self.title)
        app.add_url_rule(
            "/",
            "webio_view",
            webio_view(self.display_task),
            methods=["GET", "POST"],
        )
        app.run(debug=False, host=self.host, port=self.port)

    def display_task(self):
        """display task"""
        session.set_env(title=self.title)
        output.put_row(
            [
                pin.put_input(
                    name="url", value=pin.pin["url"], placeholder="图像数据url"
                ),
                output.put_button(
                    label="确认", onclick=self.confirm_callback, disabled=False
                ),
            ],
            size="auto",
        )
        pin.put_checkbox(
            name="cache",
            options=[{"label": "开启缓存", "value": "开启缓存", "selected": False}],
        )
        default_config = """
{
    "name": "a",
    "url_key": "href",
    "pattern": ".*?.(?:jpg|png)",
    "attrs": {}
}
"""
        pin.put_textarea(
            name="config",
            label="配置",
            value=default_config,
            rows=8,
            code={"mode": "json", "theme": "darcula"},
        )

    def confirm_callback(self):
        """confirm callback"""
        global image_urls
        url = pin.pin["url"]
        cache = pin.pin["cache"]
        config = pin.pin["config"]
        logging.info("url: {}".format(url))
        logging.info("config: {}".format(config))

        # parse config
        try:
            config = json.loads(config)
        except Exception as e:
            logging.error(e)
            output.toast(content="配置不是一个合法的json字典", duration=1)
            return

        name = config.get("name", None)
        url_key = config.get("url_key", None)
        pattern = config.get("pattern", None)
        attrs = config.get("attrs", {})
        if name is None:
            output.toast(content="配置必须包含`name`", duration=1)
            return
        elif url_key is None:
            output.toast(content="配置必须包含`url_key`", duration=1)
            return
        elif pattern is None:
            output.toast(content="配置必须包含`pattern`", duration=1)
            return
        elif not isinstance(attrs, dict):
            output.toast(content="`attrs`必须是一个字典", duration=1)
            return

        try:
            url_links = self.crawler.crawl(url, name, url_key, pattern, attrs)
            if len(url_links) == 0:
                output.toast(content="图像链接获取失败，请重新设置`url`", duration=1)
                return
            logging.info("url_links: {}".format("\n".join(url_links)))

            if cache:
                if os.path.exists(self.cache_path):
                    shutil.rmtree(self.cache_path)
                image_urls = self.crawler.download(url_links, self.cache_path)
                logging.info("image number: {}".format(len(image_urls)))
                if len(image_urls) == 0:
                    output.toast(content="图像下载失败，请重试", duration=1)
                    return
                image_urls = [
                    "static/" + os.path.basename(image_url)
                    for image_url in image_urls
                ]
            else:
                image_urls = url_links

            image_urls = sorted(image_urls)
            output.toast(content="图像数量：{}".format(len(image_urls)), duration=1)
            time.sleep(0.2)

            # 在新窗口中打开 Flask 页面
            session.run_js(
                'window.open("http://{}:{}/image", name="play image")'.format(
                    self.host, self.port
                )
            )

        except ValueError as e:
            logging.error(e)
            output.toast(content="url无效，请重新设置", duration=1)


def main():
    # set base logging config
    fmt = "[%(asctime)s - %(levelname)s - %(filename)s:%(lineno)s] %(message)s"
    logging.basicConfig(format=fmt, level=logging.INFO)

    # arg
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--host", type=str, help="host", default=get_local_ip()
    )
    parser.add_argument("--port", type=int, help="port", default=8082)
    opts = parser.parse_args()
    print(opts)

    t1 = time.time()

    title = "在线图像播放器"
    host = opts.host
    port = opts.port
    TaskManager(title=title, host=host, port=port)

    t2 = time.time()
    logging.info("total time: {}".format(t2 - t1))


if __name__ == "__main__":
    main()
